module Pod
    class Podfile
        module DSL
            # Enable Podfile Patch feature.
            # 
            # Thid feature provide the ability to override podfile in another file.
            #
            # Place a `Podfile.patch` at the same directory with Podfile. Configurate the pods
            # in patchfile just like in Podfile. It will append to original podfile, and override
            # the settings in podfile if duplicated.
            #
            # For example:
            #
            # Podfile:
            # ```
            #     target EVA do
            #         pod "A", "1.5.3", :inhibit_warning => true
            #         pod "B", :path => "some/path"
            #     end
            # ```
            #
            # Podfile.patch
            # ```
            #     target EVA do
            #         pod "A", :path => "path/to/local/A"
            #     end
            # ```
            #
            # The final result equal to 
            # ```
            #     target EVA do
            #         pod "A", :path => "path/to/local/A"
            #         pod "B", :path => "some/path"
            #     end
            # ```
            #
            # This function is base on the parsed result of podfile. It means you could do ANYTHING
            # you do in podfile, i.e. your custom pod function or other plugins.
            #
            def use_patch_file!(patchfile_path = 'Podfile.patch')
                
                @CococapodsPodfilePatch_current_patch_files ||= []
                if @CococapodsPodfilePatch_current_patch_files.include? patchfile_path
                    # Yes, you could even use it recureively, but don't use the same patch file.
                    raise "`use_patch_file!` called recursively. Most case is it's used in Patch file."
                    return 
                end
                
                @CococapodsPodfilePatch_current_patch_files << patchfile_path
                require 'cocoapods-podfile_patch/podfile_patch'
                @CococapodsPodfilePatch_patch_target = CococapodsPodfilePatch::PodfilePatch.new.load_patch_target self, patchfile_path
                @CococapodsPodfilePatch_current_patch_files.delete(patchfile_path)
            end

        end
    end
end

module Pod
    class Podfile
        old_initialize_function = instance_method(:initialize)
        define_method(:initialize) do |defined_in_file = nil, internal_hash = {}, &block|
            old_initialize_function.bind(self).(defined_in_file, internal_hash, &block)

            if not @CococapodsPodfilePatch_patch_target.nil?
                CococapodsPodfilePatch::PodfilePatch.new.merge_patch_and_original_podfile(self, @CococapodsPodfilePatch_patch_target)
            end
        end
    end
end